@@ -16,10 +16,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: input_dvd.c,v 1.214 2007/01/19 01:05:25 dgp85 Exp $
- *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 /* This file was origninally part of the xine-dvdnav project
@@ -64,21 +61,17 @@
 #endif /* WIN32 */
 
 
-#if defined(__NetBSD__) || defined(__OpenBSD__) || defined(__FreeBSD__)
+#if defined(HAVE_LINUX_CDROM_H)
+#include <linux/cdrom.h>
+#elif defined(HAVE_SYS_DVDIO_H)
 #include <sys/dvdio.h>
 #include <sys/cdio.h> /* CDIOCALLOW etc... */
-#elif defined(HAVE_LINUX_CDROM_H)
-#include <linux/cdrom.h>
 #elif defined(HAVE_SYS_CDIO_H)
 #include <sys/cdio.h>
-#else
-
-#ifdef WIN32
+#elif defined(WIN32)
 #include <io.h>                                                 /* read() */
 #else
 #warning "This might not compile due to missing cdrom ioctls"
-#endif /* WIN32 */
-
 #endif
 
 /* DVDNAV includes */
@@ -359,11 +352,14 @@ static void send_mouse_enter_leave_event(dvd_input_plugin_t *this, int direction
 }
  
 static int update_title_display(dvd_input_plugin_t *this) {
-  char ui_title[MAX_STR_LEN + 1];
-  xine_event_t uevent;
   xine_ui_data_t data;
+  xine_event_t uevent = {
+    .type = XINE_EVENT_UI_SET_TITLE,
+    .stream = this->stream,
+    .data = &data,
+    .data_length = sizeof(data)
+  };
   int tt=-1, pr=-1;
-  size_t ui_str_length=0;
   int num_tt = 0;
 
   if(!this || !(this->stream)) 
@@ -393,15 +389,15 @@ static int update_title_display(dvd_input_plugin_t *this) {
     dvdnav_get_number_of_parts(this->dvdnav, tt, &num_part);
     dvdnav_get_angle_info(this->dvdnav, &cur_angle, &num_angle);
     if(num_angle > 1) {
-      snprintf(ui_title, MAX_STR_LEN,
-               "Title %i, Chapter %i, Angle %i of %i",
-               tt,pr,cur_angle, num_angle); 
+      data.str_len = snprintf(data.str, sizeof(data.str),
+			       "Title %i, Chapter %i, Angle %i of %i",
+			       tt,pr,cur_angle, num_angle); 
        _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_NUMBER,cur_angle);
        _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_COUNT,num_angle);
     } else {
-      snprintf(ui_title, MAX_STR_LEN, 
-	       "Title %i, Chapter %i",
-	       tt,pr);
+      data.str_len = snprintf(data.str, sizeof(data.str), 
+			       "Title %i, Chapter %i",
+			       tt,pr);
        _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_NUMBER,0);
        _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_COUNT,0);
     }
@@ -410,9 +406,9 @@ static int update_title_display(dvd_input_plugin_t *this) {
     _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_CHAPTER_NUMBER,pr);
     _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_CHAPTER_COUNT,num_part);
   } else if (tt == 0 && dvdnav_menu_table[pr]) {
-    snprintf(ui_title, MAX_STR_LEN,
-             "DVD %s Menu",
-             dvdnav_menu_table[pr]);
+    data.str_len = snprintf(data.str, sizeof(data.str),
+			     "DVD %s Menu",
+			     dvdnav_menu_table[pr]);
      _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_TITLE_NUMBER,tt);
      _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_TITLE_COUNT,num_tt);
      _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_CHAPTER_NUMBER,0);
@@ -420,7 +416,8 @@ static int update_title_display(dvd_input_plugin_t *this) {
      _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_NUMBER,0);
      _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_COUNT,0);
   } else {
-    strcpy(ui_title, "DVD Menu");
+    strcpy(data.str, "DVD Menu");
+    data.str_len = strlen(data.str);
     _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_TITLE_NUMBER,0);
     _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_TITLE_COUNT,num_tt);
     _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_CHAPTER_NUMBER,0);
@@ -428,22 +425,15 @@ static int update_title_display(dvd_input_plugin_t *this) {
     _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_NUMBER,0);
     _x_stream_info_set(this->stream,XINE_STREAM_INFO_DVD_ANGLE_COUNT,0);
   }
-  ui_str_length = strlen(ui_title);
   
   if (this->dvd_name && this->dvd_name[0] &&
-      (ui_str_length + strlen(this->dvd_name) < MAX_STR_LEN)) {
-    snprintf(ui_title+ui_str_length, MAX_STR_LEN - ui_str_length, 
-	     ", %s", this->dvd_name);
+      (data.str_len + strlen(this->dvd_name) < sizeof(data.str))) {
+    data.str_len += snprintf(data.str+data.str_len, sizeof(data.str) - data.str_len, 
+			      ", %s", this->dvd_name);
   }
 #ifdef INPUT_DEBUG
-  printf("input_dvd: Changing title to read '%s'\n", ui_title);
+  printf("input_dvd: Changing title to read '%s'\n", data.str);
 #endif
-  uevent.type = XINE_EVENT_UI_SET_TITLE;
-  uevent.stream = this->stream;
-  uevent.data = &data;
-  uevent.data_length = sizeof(data);;
-  memcpy(data.str, ui_title, strlen(ui_title) + 1);
-  data.str_len = strlen(ui_title) + 1;
   xine_event_send(this->stream, &uevent);
 
   return 1;
@@ -1278,7 +1268,7 @@ static int dvd_plugin_get_optional_data (input_plugin_t *this_gen,
     if(this && this->stream && this->dvdnav) {
 
       if(!(dvdnav_is_domain_vts(this->dvdnav))) {
-	sprintf(data, "%s", "menu");
+	strcpy(data, "menu");
 	if (channel <= 0)
 	  return INPUT_OPTIONAL_SUCCESS;
 	else
@@ -1297,11 +1287,11 @@ static int dvd_plugin_get_optional_data (input_plugin_t *this_gen,
 	  sprintf(data, " %c%c", lang >> 8, lang & 0xff);
 	  /* TODO: provide long version in XINE_META_INFO_FULL_LANG */
 	else
-	  sprintf(data, " %c%c", '?', '?');
+	  strcpy(data, " ??");
 	return INPUT_OPTIONAL_SUCCESS;
       } else {
         if (channel == -1) {
-	  sprintf(data, "%s", "none");
+	  strcpy(data, "none");
 	  return INPUT_OPTIONAL_SUCCESS;
 	}
       }
@@ -1320,7 +1310,7 @@ static int dvd_plugin_get_optional_data (input_plugin_t *this_gen,
     if(this && this->stream && this->dvdnav) {
 
       if(!(dvdnav_is_domain_vts(this->dvdnav))) {
-	sprintf(data, "%s", "menu");
+	strcpy(data, "menu");
 	if (channel <= 0)
 	  return INPUT_OPTIONAL_SUCCESS;
 	else
@@ -1343,7 +1333,7 @@ static int dvd_plugin_get_optional_data (input_plugin_t *this_gen,
 	return INPUT_OPTIONAL_SUCCESS;
       } else {
 	if(channel == -1) {
-	  sprintf(data, "%s", "none");
+	  strcpy(data, "none");
 	  return INPUT_OPTIONAL_SUCCESS;
 	}
       }
@@ -1376,10 +1366,7 @@ check_solaris_vold_device(dvd_input_class_t *this)
       (volume_action = getenv("VOLUME_ACTION")) != NULL &&
       strcmp(volume_action, "insert") == 0) {
 
-    device = malloc(strlen(volume_device) + strlen(volume_name) + 2);
-    if (device == NULL)
-      return;
-    sprintf(device, "%s/%s", volume_device, volume_name);
+    asprintf(&device, "%s/%s", volume_device, volume_name);
     if (stat(device, &stb) != 0 || !S_ISCHR(stb.st_mode)) {
       free(device);
       return;
@@ -1481,7 +1468,7 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
   dvd_input_plugin_t    *this = (dvd_input_plugin_t*)this_gen;
   dvd_input_class_t     *class = (dvd_input_class_t*)this_gen->input_class;
   
-  char                  *locator;
+  char                  *locator, *locator_orig;
   char                  *title_part;
   xine_event_t           event;
   xine_cfg_entry_t       region_entry, lang_entry, cfg_entry;
@@ -1489,12 +1476,14 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
   trace_print("Called\n");
 
   /* we already checked the "dvd:/" MRL before */
-  locator = this->mrl + (sizeof("dvd:") - 1);
+  locator_orig = locator = strdup (this->mrl + (sizeof("dvd:") - 1));
 
   /* FIXME: call a generic xine-lib MRL parser here to pre-parse
    * the MRL for ?title=<title>&part=<part> stuff and to expand 
    * escaped characters properly */
-  
+
+  _x_mrl_unescape (locator);
+
   this->mode = dvd_parse_mrl(this, &locator, &title_part);
   
   if (this->mode == MODE_FAIL) {
@@ -1503,9 +1492,10 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
     _x_message(this->stream, XINE_MSG_READ_ERROR,
       /* FIXME: see FIXME in dvd_parse_try_open() */
       (strlen(locator) && !(locator[0] == '/' && locator[1] == '\0')) ? locator : class->dvd_device, NULL);
+    free (locator_orig);
     return 0;
   }
-  
+
   dvdnav_get_title_string(this->dvdnav, &this->dvd_name);
   if(this->dvd_name)
     _x_meta_info_set(this->stream, XINE_META_INFO_TITLE, this->dvd_name);
@@ -1553,6 +1543,7 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
 	      "input_dvd: Title %i is out of range (1 to %i).\n", tt, titles);
       dvdnav_close(this->dvdnav);
       this->dvdnav = NULL;
+      free (locator_orig);
       return 0;
     }
 
@@ -1566,6 +1557,7 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
 		"input_dvd: Part %i is out of range (1 to %i).\n", pr, parts);
 	dvdnav_close(this->dvdnav);
 	this->dvdnav = NULL;
+	free (locator_orig);
 	return 0;
       }
     }
@@ -1597,7 +1589,8 @@ static int dvd_plugin_open (input_plugin_t *this_gen) {
   xine_event_send(this->stream, &event);
 
   update_title_display(this);
-  
+
+  free (locator_orig);
   return 1;
 }
 
@@ -1626,14 +1619,14 @@ static input_plugin_t *dvd_class_get_instance (input_class_t *class_gen, xine_st
   if (strncasecmp (data, handled_mrl, strlen(handled_mrl) ) != 0)
     return NULL;
 
-  this = (dvd_input_plugin_t *) xine_xmalloc (sizeof (dvd_input_plugin_t));
+  this = calloc(1, sizeof (dvd_input_plugin_t));
   if (!this) {
     return NULL;
   }
 
   this->mem_stack     = 0;
   this->mem_stack_max = 1024;
-  this->mem           = xine_xmalloc(sizeof(unsigned char *) * this->mem_stack_max);
+  this->mem           = calloc(this->mem_stack_max, sizeof(unsigned char *));
   if (!this->mem) {
     free(this);
     return NULL;
@@ -1749,9 +1742,9 @@ static void *init_class (xine_t *xine, void *data) {
   dvd_input_class_t   *this;
   config_values_t     *config = xine->config;
   void                *dvdcss;
-  static char         *skip_modes[] = {"skip program", "skip part", "skip title", NULL};
-  static char         *seek_modes[] = {"seek in program chain", "seek in program", NULL};
-  static char         *play_single_chapter_modes[] = {"entire dvd", "one chapter", NULL};
+  static const char   *skip_modes[] = {"skip program", "skip part", "skip title", NULL};
+  static const char   *seek_modes[] = {"seek in program chain", "seek in program", NULL};
+  static const char   *play_single_chapter_modes[] = {"entire dvd", "one chapter", NULL};
 
   trace_print("Called\n");
 #ifdef INPUT_DEBUG
@@ -1759,7 +1752,7 @@ static void *init_class (xine_t *xine, void *data) {
   printf("input_dvd.c: config = %p\n", config);
 #endif
 
-  this = (dvd_input_class_t *) xine_xmalloc (sizeof (dvd_input_class_t));
+  this = (dvd_input_class_t *) calloc(1, sizeof (dvd_input_class_t));
   if (!this)
     return NULL;
   
@@ -1795,7 +1788,7 @@ static void *init_class (xine_t *xine, void *data) {
   {
     /* we have found libdvdcss, enable the specific config options */
     char *raw_device;
-    static char *decrypt_modes[] = { "key", "disc", "title", NULL };
+    static const char *decrypt_modes[] = { "key", "disc", "title", NULL };
     char *css_cache_default, *css_cache;
     int mode;
     
@@ -1821,8 +1814,7 @@ static void *init_class (xine_t *xine, void *data) {
 				   "playing scrambled DVDs."), 20, NULL, NULL);
     xine_setenv("DVDCSS_METHOD", decrypt_modes[mode], 0);
     
-    css_cache_default = (char *)malloc(strlen(xine_get_homedir()) + 10);
-    sprintf(css_cache_default, "%s/.dvdcss/", xine_get_homedir());
+    asprintf(&css_cache_default, "%s/.dvdcss/", xine_get_homedir());
     css_cache = config->register_filename(config, "media.dvd.css_cache_path", css_cache_default, XINE_CONFIG_STRING_IS_DIRECTORY_NAME,
 					_("path to the title key cache"),
 					_("Since cracking the copy protection of scrambled DVDs can "
