@@ -15,13 +15,11 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * Read from a tcp network stream over a lan (put a tweaked mp1e encoder the
  * other end and you can watch tv anywhere in the house ..)
  *
- * $Id: input_net.c,v 1.63 2005/02/06 15:00:36 tmattern Exp $
- *
  * how to set up mp1e for use with this plugin:
  * 
  * use mp1 to capture the live stream, e.g.
@@ -253,7 +251,7 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
   off_t n, total;
 
-  lprintf("reading %d bytes...\n", len);
+  lprintf("reading %" PRIdMAX " bytes...\n", (intmax_t)len);
 
   total=0;
   if (this->curpos < this->preview_size) {
@@ -261,7 +259,7 @@ static off_t net_plugin_read (input_plugin_t *this_gen,
     if (n > (len - total))
       n = len - total;
 
-    lprintf("%lld bytes from preview (which has %lld bytes)\n", n, this->preview_size);
+    lprintf("%" PRIdMAX " bytes from preview (which has %" PRIdMAX " bytes)\n", (intmax_t)n, (intmax_t)this->preview_size);
 
     memcpy (&buf[total], &this->preview[this->curpos], n);
     this->curpos += n;
@@ -293,7 +291,7 @@ static buf_element_t *net_plugin_read_block (input_plugin_t *this_gen,
   buf->content = buf->mem;
   buf->type = BUF_DEMUX_BLOCK;
   
-  total_bytes = net_plugin_read (this_gen, buf->content, todo);
+  total_bytes = net_plugin_read (this_gen, (char*)buf->content, todo);
 
   if (total_bytes != todo) {
     buf->free_buffer (buf);
@@ -367,7 +365,7 @@ static off_t net_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
 }
 
 
-static char* net_plugin_get_mrl (input_plugin_t *this_gen) {
+static const char* net_plugin_get_mrl (input_plugin_t *this_gen) {
   net_input_plugin_t *this = (net_input_plugin_t *) this_gen;
 
   return this->mrl;
@@ -413,6 +411,8 @@ static int net_plugin_open (input_plugin_t *this_gen ) {
   char *filename;
   char *pptr;
   int port = 7658;
+  int toread = MAX_PREVIEW_SIZE;
+  int trycount = 0;
 
   filename = this->host_port;
   pptr=strrchr(filename, ':');
@@ -431,11 +431,15 @@ static int net_plugin_open (input_plugin_t *this_gen ) {
   /*
    * fill preview buffer
    */
+  while ((toread > 0) && (trycount < 10)) {
 #ifndef WIN32
-  this->preview_size = read (this->fh, this->preview, MAX_PREVIEW_SIZE);
+    this->preview_size += read (this->fh, this->preview + this->preview_size, toread);
 #else
-  this->preview_size = recv (this->fh, this->preview, MAX_PREVIEW_SIZE, 0);
+    this->preview_size += recv (this->fh, this->preview + this->preview_size, toread, 0);
 #endif
+    trycount++;
+    toread = MAX_PREVIEW_SIZE - this->preview_size;
+  }
 
   this->curpos       = 0;
 
@@ -476,7 +480,7 @@ static input_plugin_t *net_class_get_instance (input_class_t *cls_gen, xine_stre
     return NULL;
   }
 
-  this = xine_xmalloc(sizeof(net_input_plugin_t));
+  this = calloc(1, sizeof(net_input_plugin_t));
   this->mrl           = strdup(mrl);
   this->host_port     = strdup(filename);
   this->stream        = stream;
@@ -506,7 +510,7 @@ static input_plugin_t *net_class_get_instance (input_class_t *cls_gen, xine_stre
  *  net plugin class
  */
  
-static char *net_class_get_description (input_class_t *this_gen) {
+static const char *net_class_get_description (input_class_t *this_gen) {
 	return _("net input plugin as shipped with xine");
 }
 
@@ -524,7 +528,7 @@ static void *init_class (xine_t *xine, void *data) {
 
   net_input_class_t  *this;
 
-  this         = (net_input_class_t *) xine_xmalloc(sizeof(net_input_class_t));
+  this         = calloc(1, sizeof(net_input_class_t));
   this->config = xine->config;
   this->xine   = xine;
 
@@ -543,9 +547,9 @@ static void *init_class (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
-plugin_info_t xine_plugin_info[] = {
+const plugin_info_t xine_plugin_info[] EXPORTED = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 16, "tcp", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 17, "tcp", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
