@@ -1,5 +1,5 @@
-/* 
- * Copyright (C) 2000-2002 the xine project
+/*
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -15,7 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  *
  * network buffering control
  */
@@ -27,10 +27,14 @@
 
 typedef struct nbc_s nbc_t;
 
-nbc_t *nbc_init (xine_t *xine);
+nbc_t *nbc_init (xine_stream_t *xine) XINE_MALLOC;
 
-void nbc_check_buffers (nbc_t *this);
+void nbc_check_buffers (nbc_t *this) XINE_DEPRECATED;
 
 void nbc_close (nbc_t *this);
 
+void nbc_set_high_water_mark(nbc_t *this, int value) XINE_DEPRECATED;
+
+void nbc_set_low_water_mark(nbc_t *this, int value) XINE_DEPRECATED;
+
 #endif
