@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2006 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -15,10 +15,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
- *
- * $Id: video_overlay.c,v 1.39 2005/09/25 00:44:04 miguelfreitas Exp $
- *
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
  */
 
 #include <stdlib.h>
@@ -51,6 +48,8 @@ typedef struct video_overlay_showing_s {
 typedef struct video_overlay_s {
   video_overlay_manager_t   video_overlay;
   
+  xine_t                   *xine;
+
   pthread_mutex_t           events_mutex;  
   video_overlay_events_t    events[MAX_EVENTS];
   pthread_mutex_t           objects_mutex;  
@@ -58,7 +57,6 @@ typedef struct video_overlay_s {
   pthread_mutex_t           showing_mutex;
   video_overlay_showing_t   showing[MAX_SHOWING];  
   int                       showing_changed;
-  xine_t                   *xine;
 } video_overlay_t;
 
 
@@ -201,7 +199,7 @@ static void video_overlay_reset (video_overlay_t *this) {
   pthread_mutex_lock (&this->events_mutex);
   for (i=0; i < MAX_EVENTS; i++) {
     if (this->events[i].event == NULL) {
-      this->events[i].event = xine_xmalloc (sizeof(video_overlay_event_t));
+      this->events[i].event = calloc(1, sizeof(video_overlay_event_t));
 #ifdef LOG_DEBUG
       printf ("video_overlay: MALLOC2: this->events[%d].event %p, len=%d\n",
 	      i,
@@ -287,10 +285,18 @@ static int32_t video_overlay_add_event(video_overlay_manager_t *this_gen,  void
     }
     
     if( event->object.overlay ) {
-      this->events[new_event].event->object.overlay = xine_xmalloc (sizeof(vo_overlay_t));
+      int i;
+      for(i = 0; i < OVL_PALETTE_SIZE; i++) {
+	if(event->object.overlay->trans[i] >= OVL_MAX_OPACITY)
+	  event->object.overlay->trans[i] = OVL_MAX_OPACITY;
+	if(event->object.overlay->hili_trans[i] >= OVL_MAX_OPACITY)
+	  event->object.overlay->hili_trans[i] = OVL_MAX_OPACITY;
+      }
+
+      this->events[new_event].event->object.overlay = calloc(1, sizeof(vo_overlay_t));
       xine_fast_memcpy(this->events[new_event].event->object.overlay, 
            event->object.overlay, sizeof(vo_overlay_t));
-    
+
       /* We took the callers rle and data, therefore it will be our job to free it */
       /* clear callers overlay so it will not be freed twice */
       memset(event->object.overlay,0,sizeof(vo_overlay_t));
@@ -347,8 +353,10 @@ static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
     handle=this->events[this_event].event->object.handle;
 #ifdef LOG_DEBUG
     printf ("video_overlay: video_overlay_event: handle = %d\n", handle);
-    assert(handle >=0);
 #endif
+    _x_assert(handle >= 0);
+    if ( handle < 0 ) break;
+
     switch( this->events[this_event].event->event_type ) {
       case OVERLAY_EVENT_SHOW:
 #ifdef LOG_DEBUG
@@ -384,6 +392,8 @@ static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
 #endif
         /* free any overlay associated with this event */
         if (this->events[this_event].event->object.overlay != NULL) {
+          if( this->events[this_event].event->object.overlay->rle != NULL )
+            free( this->events[this_event].event->object.overlay->rle );
           free(this->events[this_event].event->object.overlay);
           this->events[this_event].event->object.overlay = NULL; 
         }
@@ -395,9 +405,11 @@ static int video_overlay_event( video_overlay_t *this, int64_t vpts ) {
         printf ("video_overlay: FREE SPU NOW\n");
 #endif
         /* free any overlay associated with this event */
-        if (this->events[this_event].event->object.overlay != NULL) {
+        if( this->events[this_event].event->object.overlay != NULL) {   
+          if( this->events[this_event].event->object.overlay->rle != NULL )
+            free( this->events[this_event].event->object.overlay->rle );
           free(this->events[this_event].event->object.overlay);
-          this->events[this_event].event->object.overlay = NULL; 
+          this->events[this_event].event->object.overlay = NULL;
         }
         /* this avoid removing this_event from the queue
          * (it will be removed at the end of this loop) */
@@ -575,7 +587,7 @@ video_overlay_manager_t *_x_video_overlay_new_manager (xine_t *xine) {
 
   video_overlay_t *this;
 
-  this = (video_overlay_t *) xine_xmalloc (sizeof (video_overlay_t));
+  this = (video_overlay_t *) calloc(1, sizeof(video_overlay_t));
 
   this->xine                              = xine;
   this->video_overlay.init                = video_overlay_init;
