@@ -29,7 +29,6 @@
 #include "xineutils.h"
 #else
 #define lprintf(...)
-#define xine_xmalloc malloc
 #endif
 #include "xmllexer.h"
 #include <stdio.h>
@@ -411,6 +410,8 @@ int lexer_get_token_d(char ** _tok, int * _tok_size, int fixed) {
 	  case '\"': /* " */
 	  case ' ':
 	  case '\t':
+	  case '\n':
+	  case '\r':
 	  case '=':
 	  case '/':
 	    tok[tok_pos] = '\0';
@@ -532,7 +533,7 @@ static struct {
 
 char *lexer_decode_entities (const char *tok)
 {
-  char *buf = xine_xmalloc (strlen (tok) + 1);
+  char *buf = calloc (strlen (tok) + 1, sizeof(char));
   char *bp = buf;
   char c;
 
